#include "header\gba.h"
#include "header\bg.h"
#include "header\timer.h"
#include "header\dma.h"
#include "header\irq.h"
#include "header\sound.h"
#include "header\sprite.h"
#include "header\blend.h"

#include "header\font\BGFont.h"
//#include "header\font\BitMapFont.h"

#include "lib.h"
#include "syscall.h"
#include "resource.h"

//Mode0
//BG0 
//BG3 pl

//---------------------------------------------------------------------------

//
#define BG_CHR_CX				8		//BGLN^[̑傫
#define BG_CHR_CY				8		
#define SCROLL_WAIT				2		//XN[EFCg
#define UNIT_MAX_CNT			20		//G@̍őo^
#define SCRIPT_MAX_LEN			10
#define BULLET_MAX_CNT			30
#define BULLET_SPEED			3


//is
#define STATE_INIT				1
#define STATE_SCROLL			2
#define STATE_UNIT				3
#define STATE_ETC				4

//XvCgԍ
#define SPRITE_CHR_NULL			0
#define SPRITE_CHR_KUTEPO3		2		//Nei|}III
#define SPRITE_CHR_KARUBARI		4		//Jo
#define SPRITE_CHR_CELLTE		6		//ZeBbN
#define SPRITE_CHR_ANSATE		8		//AT[e
#define SPRITE_CHR_XIX			10		//XIX
#define SPRITE_CHR_SEMIYON		12		//Z~
#define SPRITE_CHR_BENEDIKU		14		//xlfBNV
#define SPRITE_CHR_GAA			16		//K[
#define SPRITE_CHR_PBENEDIKU	18		//p[pExlfBNV
#define SPRITE_CHR_KASA			20		//P
#define SPRITE_CHR_KARASIN		22		//JV
#define SPRITE_CHR_EKUROLL		24		//GN[Eo
#define SPRITE_CHR_ISIZUKAN		26		//CVYJ
#define SPRITE_CHR_F52			28		//F52
#define SPRITE_CHR_STORAKU		30		//XgN`[E{[
#define SPRITE_CHR_KERUTES		32		//PeX
#define SPRITE_CHR_MAX_CNT		(32/2)	//XvCgv

#define SPRITE_CHR_BULLET		72

//XNvg`
#define SPI_UP					1
#define SPI_UP_LEFT				2
#define SPI_UP_RIGHT			4
#define SPI_LEFT				8
#define SPI_NONE				16
#define SPI_RIGHT				32
#define SPI_DOWN				64
#define SPI_DOWN_LEFT			128
#define SPI_DOWN_RIGHT			256
#define SPI_SHOT1				512
#define SPI_SHOT2				1024
#define SPI_SHOT3				2048
#define SPI_BJMP				4096
//---------------------------------------------------------------------------
//BG
typedef struct {
	u8 Chr;
	u8 Map;
	u8 Col;
} ST_BG;

//Stage
typedef struct {
	u16  BGCx;
	u16  BGCy;
	u16  BGhofs;
	u16  BGvofs;
	u16* pMap;
	u16* pMapEnd;
	u16* pMapStart;
	u16* pMapData;
	u16* pChrData;
	s16  lineNowCnt;
	s16  lineEndCnt;
	s16  readNowCnt;
	s16  readEndCnt;
	s16  scrollWaitCnt;
} ST_STAGE;

//@
typedef struct {
	s16 x;
	s16 y;
	u16 cx;
	u16 cy;
	s16 moveX;
	s16 moveY;
	s16 minX;			//ړ
	s16 minY;
	s16 maxX;
	s16 maxY;
	u16 sprNo;
	u16 sprTile;
	u16 sprSize;
	u16 sprForm;
	u8  key;			//ړp
	s16 scoure;			//XRA
} ST_MYSHIP;

//G@
typedef struct {
	bool sw;			//^XN̓o^
	s16  x;
	s16  y;
	u16  cx;
	u16  cy;
	u16  sprNo;
	u16  sprTile;
	u16  sprSize;
	u16  sprForm;
	s16* pCode;			//vOJE^iEIP|C^j
	s16  code;			//ݎs̃R[h
	s16  data;			//ݎs̓񐔂̃JEg
	u16  score;
} ST_UNIT;

//G@XNvg
typedef struct {
	s16 code[SCRIPT_MAX_LEN];
	s16 data[SCRIPT_MAX_LEN];
} ST_UNIT_SPI;

//XvCgǗ
typedef struct {
	s16  cnt;
	bool sw[MAX_SPRITE];
} ST_SPR_CON;

//e
typedef struct {
	bool sw;		//
	f32  x;
	f32  y;
	f32  txv;		//ړ
	f32  tyv;
	u8   angle;		//px
	f32  speed;		//Xs[h
	u16  cx;
	u16  cy;
	u16  sprNo;
	u16  sprTile;
	u16  sprSize;
	u16  sprForm;
}ST_BULLET;

//---------------------------------------------------------------------------
ST_BG       BG0;
ST_BG       BG3;
ST_STAGE    Stage;
ST_MYSHIP   MyShip;
ST_UNIT     Unit[UNIT_MAX_CNT];
ST_BULLET   Bullet[BULLET_MAX_CNT];
ST_UNIT_SPI Spit[SPRITE_CHR_MAX_CNT];
ST_SPR_CON  SprControl;

u8 State;			//istO

//---------------------------------------------------------------------------
void StateInit();
void StateScroll();
void StateUnit();
void StateEtc();

void InitBG();
void InitSprite();
void InitStage();
void InitMyShip();
void InitSprControl();
void InitUnit();
void InitBullet();
void InitScript();

void NextScrollBG();
bool ChkNextScroll();
void NextLoadMapLine();
void NextLoadChrLine();
bool ChkNextLoadLine();

void ProcMyShip();
void ProcUnit();
void ProcBullet();

void AddUnit(s16 x, s16 y, u16 sprTile);
void AddBullet(s16 x, s16 y, u8 angle);

void MoveUnit(ST_MYSHIP* p);
void DrawUnit(ST_MYSHIP* p);
bool ChkMoveUnit(ST_MYSHIP* p);

void MoveBullet(ST_BULLET* p);
void DrawBullet(ST_BULLET* p);
bool ChkMoveBullet(ST_BULLET* p);

void ScriptExec(ST_UNIT* p);

u16  SprRelease();
void SprFree(u16 no);

void BulletSetShot1(s16 x, s16 y);
void BulletSetShot2(s16 x, s16 y);
void BulletSetShot3(s16 x, s16 y);
u8   BulletCalcAngle(s16 x, s16 y);

void DrawBGScore();

//---------------------------------------------------------------------------
int main(void)
{
	State = STATE_INIT;

	for(;;)
	{
		WaitForVsync();

		switch(State)
		{
			case STATE_INIT:    StateInit();   break;
			case STATE_SCROLL:  StateScroll(); break;
			case STATE_UNIT:    StateUnit();   break;
			case STATE_ETC:     StateEtc();    break;
			default:
				DebugMessage((u8*)"Err: Main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
void StateInit()
{
	//EFCgRg[
	REG_WSCNT = CST_ROM0_1ST_3WAIT | CST_ROM0_2ND_1WAIT;

	InitBG();
	InitSprite();

	InitStage();
	InitMyShip();
	InitSprControl();
	InitUnit();
	InitScript();
	InitBullet();

	State = STATE_SCROLL;
	SetMode(MODE_0 | BG0_ENABLE | BG3_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);
}
//-----------------------------------------------------------------------------
void InitStage()
{
	Stage.BGCx          = 32;
	Stage.BGCy          = 20;
	Stage.BGhofs        = 0;
	Stage.BGvofs        = 96;
	Stage.pMap          = (u16*)MEM_BG_MAP(BG3.Map) + (2048-2) / 2;
	Stage.pMapEnd       = (u16*)MEM_BG_MAP(BG3.Map) + (2048-2) / 2;
	Stage.pMapStart     = (u16*)MEM_BG_MAP(BG3.Map);
	Stage.pMapData      = (u16*)&map_raw_bin + (MAP_RAW_SIZE-2) / 2;
	Stage.pChrData      = (u16*)&chr_raw_bin + (CHR_RAW_SIZE-2) / 2;
	Stage.lineNowCnt    = 160;
	Stage.lineEndCnt    = 1024;
	Stage.readNowCnt    = 0;
	Stage.readEndCnt    = 128;
	Stage.scrollWaitCnt = 0;

	//ʂ̏ݒ
	s16 i;
	for(i=0; i<Stage.BGCy+2; i++)
	{
		NextLoadMapLine();
		NextLoadChrLine();
		Stage.readNowCnt++;
	}
	REG_BG3VOFS = Stage.BGvofs;

	//bZ[W
	DrawBGScore();
}
//-----------------------------------------------------------------------------
void InitMyShip()
{
	//L[
	KeyInit();
	MyShip.key = 0x00;

	MyShip.cx      = 16;
	MyShip.cy      = 16;
	MyShip.x       = (SCREEN_CX/2 - MyShip.cx/2);
	MyShip.y       = (SCREEN_CY/2 - MyShip.cy/2);
	MyShip.moveX   = 5;
	MyShip.moveY   = 5;
	MyShip.minX    = 0;
	MyShip.minY    = 0;
	MyShip.maxX    = SCREEN_CX - MyShip.cx;
	MyShip.maxY    = SCREEN_CY - MyShip.cy;
	MyShip.sprNo   = 0;
	MyShip.sprTile = 2;
	MyShip.sprSize = SP_SIZE_16;
	MyShip.sprForm = SP_SQUARE;
	MyShip.scoure  = 0;

	SpriteSetSize(MyShip.sprNo, MyShip.sprSize, MyShip.sprForm, SP_COLOR_16);
	SpriteSetChr (MyShip.sprNo, MyShip.sprTile);
	SpriteMove   (MyShip.sprNo, MyShip.x, MyShip.y);
}
//-----------------------------------------------------------------------------
void NextLoadMapLine()
{
	//}bṽItZbgJnʒu菬ꍇAIʒuɃZbg܂
	if(Stage.pMap < Stage.pMapStart)
	{
		Stage.pMap = Stage.pMapEnd;
	}

	//}bvf[^̃Rs[
	s16 i;
	for(i=0; i<Stage.BGCx; i++)
	{
		*Stage.pMap-- = *Stage.pMapData--;
	}
}
//-----------------------------------------------------------------------------
void StateScroll()
{
	ST_STAGE* p = &Stage;

	//XN[EFCg
	p->scrollWaitCnt = ++p->scrollWaitCnt % SCROLL_WAIT;
	if(p->scrollWaitCnt != 0)
	{
		goto endFnc;
	}

	//}bṽXN[
	if( ChkNextScroll() == TRUE )
	{
		NextScrollBG();
		p->lineNowCnt++;
	}

	//Xe[Wf[^ʂ̏㕔i\jɃRs[
	if( ChkNextLoadLine() == TRUE )
	{
		NextLoadMapLine();
		NextLoadChrLine();
		p->readNowCnt++;
	}

endFnc:

	State = STATE_UNIT;
}
//-----------------------------------------------------------------------------
void NextScrollBG()
{
	Stage.BGvofs = --Stage.BGvofs & 0x1ff;
//	REG_BG3HOFS = Stage.BGhofs;		//x
	REG_BG3VOFS = Stage.BGvofs;		//y
}
//-----------------------------------------------------------------------------
void ProcMyShip()
{
	ST_MYSHIP* p = &MyShip;
	u8 Key       = KeyInput();

	//͂ĂȂ`FbN܂
	if(Key == 0x00)
	{
		goto endFnc;
	}

	//ړ\`FbN܂
	p->key = Key;
	if( ChkMoveUnit(p) == TRUE )
	{
		//@̈ړ
		MoveUnit(p);
		DrawUnit(p);
	}

endFnc:

	State = STATE_UNIT;
}
//---------------------------------------------------------------------------
//ړ͈ړ\`FbN܂
u8 ChkMoveUnit(ST_MYSHIP* p)
{
	s16 tx1 = p->x;
	s16 ty1 = p->y;

	if(p->key & KEY_UP   ) ty1 -= p->moveY;
	if(p->key & KEY_DOWN ) ty1 += p->moveY;
	if(p->key & KEY_LEFT ) tx1 -= p->moveX;
	if(p->key & KEY_RIGHT) tx1 += p->moveX;

	if(tx1< p->minX || ty1< p->minY) return FALSE;
	if(tx1>=p->maxX || ty1>=p->maxY) return FALSE;

	return TRUE;
}
//---------------------------------------------------------------------------
//LN^̈ړ
void MoveUnit(ST_MYSHIP* p)
{
	if(p->key & KEY_UP   ) p->y-=p->moveY;
	if(p->key & KEY_DOWN ) p->y+=p->moveY;
	if(p->key & KEY_LEFT ) p->x-=p->moveX;
	if(p->key & KEY_RIGHT) p->x+=p->moveX;
}
//---------------------------------------------------------------------------
void DrawUnit(ST_MYSHIP* p)
{
	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
void NextLoadChrLine()
{
	s16 x;
	u16 sprTile;
	
	for(x=Stage.BGCx; x>0; x--)
	{
		sprTile = *Stage.pChrData--;

		if(sprTile == 0x0000)
		{
			continue;
		}
		switch(sprTile)
		{
			case SPRITE_CHR_KUTEPO3:
			case SPRITE_CHR_KARUBARI:
			case SPRITE_CHR_CELLTE:
			case SPRITE_CHR_ANSATE:
			case SPRITE_CHR_XIX:
			case SPRITE_CHR_SEMIYON:
			case SPRITE_CHR_BENEDIKU:
			case SPRITE_CHR_GAA:
			case SPRITE_CHR_PBENEDIKU:
			case SPRITE_CHR_KASA:
			case SPRITE_CHR_KARASIN:
			case SPRITE_CHR_EKUROLL:
			case SPRITE_CHR_ISIZUKAN:
			case SPRITE_CHR_F52:
			case SPRITE_CHR_STORAKU:
			case SPRITE_CHR_KERUTES:

				//G@^XNo^܂
				AddUnit(BG_CHR_CX*x, BG_CHR_CY*-2, sprTile);
				break;
			default:
				break;
		}
	}
}
//---------------------------------------------------------------------------
bool ChkNextLoadLine()
{
	ST_STAGE* p = &Stage;

	//}bvf[^̓ǂݍ݂IĂ邩`FbN܂
	if(p->readNowCnt >= p->readEndCnt)
	{
		return FALSE;
	}

	//8hbgXN[`FbN܂
	if(p->lineNowCnt % BG_CHR_CY != 0)
	{
		return FALSE;
	}

	return TRUE;
}
//---------------------------------------------------------------------------
void InitUnit()
{
	s16 i;
	for(i=0; i<UNIT_MAX_CNT; i++)
	{
		ST_UNIT* p = &Unit[i];

		p->sw       = FALSE;
		p->x        = 0;
		p->y        = 0;
		p->cx       = 16;
		p->cy       = 16;
		p->sprNo    = 0;
		p->sprTile  = 0;
		p->sprSize  = SP_SIZE_16;
		p->sprForm  = SP_SQUARE;

		p->pCode    = NULL;
		p->code     = 0;
		p->data     = 0;
	}
}
//---------------------------------------------------------------------------
void AddUnit(s16 x, s16 y, u16 sprTile)
{
	u16 sprNo = SprRelease();
	if(sprNo == 0)
	{
		return;
	}

	s16 i;
	for(i=0; i<UNIT_MAX_CNT; i++)
	{
		if(Unit[i].sw == FALSE)
		{
			ST_UNIT* p = &Unit[i];

			p->sw       = TRUE;
			p->x        = x;
			p->y        = y;
			p->sprNo    = sprNo;
			p->sprTile  = sprTile;
			p->pCode    = &Spit[sprTile/2].code[0];
			p->code     = 0;
			p->data     = 0;
			p->score    = 0;

			SpriteSetSize(p->sprNo, p->sprSize, p->sprForm, SP_COLOR_16);
			SpriteSetChr (p->sprNo, p->sprTile);
			SpriteMove   (p->sprNo, p->x, p->y);

			return;
		}
	}
}
//---------------------------------------------------------------------------
void AddBullet(s16 x, s16 y, u8 angle)
{
	u16 sprNo = SprRelease();
	if(sprNo == 0)
	{
		return;
	}

	s16 i;
	for(i=0; i<BULLET_MAX_CNT; i++)
	{
		if(Bullet[i].sw == FALSE)
		{
			ST_BULLET* p = &Bullet[i];

			p->sw       = TRUE;
			p->x        = x;
			p->y        = y;
			p->cx       = 8;
			p->cy       = 8;
			p->txv      = BULLET_SPEED * _Cos(angle);
			p->tyv      = BULLET_SPEED * _Sin(angle);
			p->angle    = angle;
			p->speed    = BULLET_SPEED;
			p->sprNo    = sprNo;
			p->sprTile  = SPRITE_CHR_BULLET;
			p->sprSize  = SP_SIZE_8;
			p->sprForm  = SP_SQUARE;

			SpriteSetSize(p->sprNo, p->sprSize, p->sprForm, SP_COLOR_16);
			SpriteSetChr (p->sprNo, p->sprTile);
			SpriteMove   (p->sprNo, p->x, p->y);

			return;
		}
	}
}
//---------------------------------------------------------------------------
void InitSprControl()
{
	SprControl.cnt = 0;

	s16 i;
	for(i=0; i<MAX_SPRITE; i++)
	{
		SprControl.sw[i] = FALSE;
	}
}
//---------------------------------------------------------------------------
u16 SprRelease()
{
	s16 i;
	for(i=1; i<MAX_SPRITE; i++)
	{
		if(SprControl.sw[i] == FALSE)
		{
			SprControl.sw[i] = TRUE;
			SprControl.cnt++;

			return i;
		}
	}

	dprintf("too many Sprite\n");
	return 0;
}
//---------------------------------------------------------------------------
void SprFree(u16 no)
{
	SprControl.sw[no] = FALSE;
	SprControl.cnt--;
}
//---------------------------------------------------------------------------
void DrawBGScore()
{
	char msg[40];
	bprintf(msg, "Sky Bruiser        SCORE:%5d", MyShip.scoure);
	BGDrawStr(BG0.Map, 0, 0, (u8*)msg);
}
//---------------------------------------------------------------------------
void StateEtc()
{
	DrawBGScore();
}
//---------------------------------------------------------------------------
void StateUnit()
{
	ProcMyShip();
	ProcUnit();
	ProcBullet();

	State = STATE_SCROLL;
}
//---------------------------------------------------------------------------
bool ChkNextScroll()
{
	ST_STAGE* p = &Stage;

	if(p->lineNowCnt >= p->lineEndCnt)
	{
		return FALSE;
	}

	return TRUE;
}
//---------------------------------------------------------------------------
void InitScript()
{
	ST_UNIT_SPI* p;
	
	//XIX
	p = &Spit[SPRITE_CHR_XIX/2];

	p->code[0] = SPI_DOWN;
	p->data[0] = 64;
	p->code[1] = SPI_SHOT1;
	p->data[1] = 1;
	p->code[2] = SPI_DOWN_RIGHT;
	p->data[2] = 2;
	p->code[3] = SPI_BJMP;
	p->data[3] = 2;

	//Jo
	p = &Spit[SPRITE_CHR_KARUBARI/2];

	p->code[0] = SPI_DOWN;
	p->data[0] = 1;
	p->code[1] = SPI_NONE;
	p->data[1] = 1;
	p->code[2] = SPI_BJMP;
	p->data[2] = 3;

	//Z~
	p = &Spit[SPRITE_CHR_SEMIYON/2];

	p->code[0] = SPI_DOWN;
	p->data[0] = 20;
	p->code[1] = SPI_SHOT2;
	p->data[1] = 1;
	p->code[2] = SPI_BJMP;
	p->data[2] = 3;

	//ZeBbN
	p = &Spit[SPRITE_CHR_CELLTE/2];

	p->code[0] = SPI_DOWN;
	p->data[0] = 1;
	p->code[1] = SPI_NONE;
	p->data[1] = 1;
	p->code[2] = SPI_BJMP;
	p->data[2] = 3;

	//XgN`[E{[
	p = &Spit[SPRITE_CHR_STORAKU/2];

	p->code[0] = SPI_DOWN;
	p->data[0] = 32;
	p->code[1] = SPI_BJMP;
	p->data[1] = 1;
}
//---------------------------------------------------------------------------
void ScriptExec(ST_UNIT* p)
{
StartFnc:

	if(p->data <= 0)
	{
		p->code = *(p->pCode);
		p->data = *(p->pCode + SCRIPT_MAX_LEN);
		p->pCode++;
	}

	switch(p->code)
	{
		//ړ
 		case SPI_UP:            p->y--;         break;
		case SPI_UP_LEFT:       p->y--; p->x--; break;
		case SPI_UP_RIGHT:      p->y--; p->x++; break;
		case SPI_LEFT:          p->x--;         break;
		case SPI_NONE:                          break;	//Ȃ
		case SPI_RIGHT:         p->x++;         break;
		case SPI_DOWN:          p->y++;         break;
		case SPI_DOWN_LEFT:     p->y++; p->x--; break;
		case SPI_DOWN_RIGHT:    p->y++; p->x++; break;

		//e
		case SPI_SHOT1:
			BulletSetShot1(p->x, p->y);
			p->data   = 0;
			goto StartFnc;
			break;

		case SPI_SHOT2:
			BulletSetShot2(p->x, p->y);
			p->data   = 0;
			goto StartFnc;
			break;

		case SPI_SHOT3:
			BulletSetShot3(p->x, p->y);
			p->data   = 0;
			goto StartFnc;
			break;

		//
		case SPI_BJMP:
			p->pCode -= p->data;
			p->data   = 0;
			goto StartFnc;
			break;

		default:
			DebugMessage((u8*)"Err: ScriptExec p->code");
			break;
	}
	p->data--;

	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
void InitSprite()
{
	SpriteInit();
	SpriteSetPal16((u16*)&spr_pal_bin);
	SpriteSetData ((u16*)&spr_img_bin, 4096);

	s16 i;
	for(i=0; i<MAX_SPRITE; i++)
	{
		SpriteSetPriority(i, 1);
	}
}
//---------------------------------------------------------------------------
void InitBG()
{
	//BG
	//BG0 
	//BG3 n`

	BG0.Chr = 1;
	BG0.Map = 0;
	BG0.Col = 0;

	BG3.Chr = 2;
	BG3.Map = 1;
	BG3.Col = 1;

	REG_BG0CNT = (BG_SIZEA_256_256     | BG_COLOR_16         |
				  BG_CHARBASE(BG0.Chr) | BG_MAPBASE(BG0.Map) | 0);

	REG_BG3CNT = (BG_SIZEA_256_256     | BG_COLOR_16         |
				  BG_CHARBASE(BG3.Chr) | BG_MAPBASE(BG3.Map) | 3);

	BGSetPal16  (BG0.Col, (u16*)&BGFont_palette);
	BGSetChrData(BG0.Chr, (u16*)&BGFont_data, BGFont_DATA_SIZE);

	BGSetPal16  (BG3.Col, (u16*)&bg_pal_bin);
	BGSetChrData(BG3.Chr, (u16*)&bg_img_bin, 16384);
}
//---------------------------------------------------------------------------
void ProcUnit()
{
	s16 i;
	for(i=1; i<MAX_SPRITE; i++)
	{
		if(Unit[i].sw == TRUE)
		{
			//XNvg̎s
			ScriptExec(&Unit[i]);

			//ʔ͈̓`FbN
			if(Unit[i].y > SCREEN_CY)
			{
				//
				Unit[i].sw = FALSE;
				SprFree(Unit[i].sprNo);
			}
		}
	}
}
//---------------------------------------------------------------------------
//e
void BulletSetShot1(s16 x, s16 y)
{
	u8 angle = BulletCalcAngle(x, y);
	AddBullet(x, y, angle);
}
//---------------------------------------------------------------------------
//R̒e
void BulletSetShot2(s16 x, s16 y)
{
	u8 angle = BulletCalcAngle(x, y);

	AddBullet(x, y, angle);
	AddBullet(x, y, angle + 10);
	AddBullet(x, y, angle - 10);
}
//---------------------------------------------------------------------------
//PU̒e
void BulletSetShot3(s16 x, s16 y)
{
	s16 ang;
	for(ang=0; ang<256; ang+=16)
	{
		AddBullet(x, y, ang);
	}
}
//---------------------------------------------------------------------------
//@Ƃ̊px߂
u8 BulletCalcAngle(s16 x, s16 y)
{
	return GetAngle(MyShip.x - x, MyShip.y - y);
}
//---------------------------------------------------------------------------
void ProcBullet()
{
	s16 i;
	for(i=0; i<BULLET_MAX_CNT; i++)
	{
		if(Bullet[i].sw == TRUE)
		{
			if( ChkMoveBullet(&Bullet[i]) != TRUE )
			{
				Bullet[i].sw = FALSE;
				continue;
			}

			MoveBullet(&Bullet[i]);
		}
	}
}
//---------------------------------------------------------------------------
bool ChkMoveBullet(ST_BULLET* p)
{
	s16 tx1 = p->x;
	s16 ty1 = p->y;

	if(tx1< 0 - p->cx || ty1< 0 - p->cy) return FALSE;
	if(tx1>=SCREEN_CX || ty1>=SCREEN_CY) return FALSE;

	return TRUE;
}
//---------------------------------------------------------------------------
void MoveBullet(ST_BULLET* p)
{
	p->x += p->txv;
	p->y += p->tyv;
	DrawBullet(p);
}
//---------------------------------------------------------------------------
void DrawBullet(ST_BULLET* p)
{
	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
void InitBullet()
{
	s16 i;
	for(i=0; i<BULLET_MAX_CNT; i++)
	{
		ST_BULLET* p = &Bullet[i];

		p->sw      = FALSE;
		p->x       = 0;
		p->y       = 0;
		p->txv     = 0;
		p->tyv     = 0;
		p->angle   = 0;
		p->speed   = 0;
		p->cx      = 0;
		p->cy      = 0;
		p->sprNo   = 0;
		p->sprTile = 0;
		p->sprSize = 0;
		p->sprForm = 0;
	}
}
